<?php
/*
  Plugin Name: WP Frontend Admin
  Plugin URI: https://wpfrontendadmin.com
  Description: Display wp-admin pages on the frontend using a shortcode.
  Version: 1.5.1
  Author: JoseVega
  Author Email: josevega@wpfrontendadmin.com
  Author URI: https://wpfrontendadmin.com
  License:

  Copyright 2018 JoseVega (josevega@vegacorp.me)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2, as
  published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 */

if (!defined('WPFA_IS_EMBED_MODE')) {
	define('WPFA_IS_EMBED_MODE', strpos(wp_normalize_path(__FILE__), 'plugins/display-admin-page') === false);
}
if (!WPFA_IS_EMBED_MODE) {
	require 'inc/freemius-init.php';
}
require 'vendor/vg-plugin-sdk/index.php';
if (!class_exists('VG_Admin_To_Frontend')) {

	class VG_Admin_To_Frontend {

		static private $instance = false;
		static $textname = 'vg_admin_to_frontend';
		static $dir = __DIR__;
		static $file = __FILE__;
		static $version = '1.5.1';
		static $name = 'Frontend Admin';
		static $main_admin_id_key = 'vgfa_admin_main_admin_id';
		var $allowed_urls = array();
		var $base_page_id = 'vgca_base_page_id';

		private function __construct() {
			
		}

		function get_upgrade_url() {
			$url = ( function_exists('dapof_fs') ) ? dapof_fs()->checkout_url(WP_FS__PERIOD_ANNUALLY, true, array(
						'licenses' => is_multisite() ? 'unlimited' : 1,
						'billing_cycle' => is_multisite() ? 'monthly' : WP_FS__PERIOD_ANNUALLY,
					)) : 'https://wpfrontendadmin.com/go/start-free-trial-wpadmin';
			return $url;
		}

		function is_main_admin_id($user_id = null) {
			if (!$user_id) {
				$user_id = get_current_user_id();
			}
			if (!empty($_GET['vgfa_is_main_admin']) && current_user_can('manage_options')) {
				update_option(VG_Admin_To_Frontend::$main_admin_id_key, $user_id);
			}
			return (int) get_option(VG_Admin_To_Frontend::$main_admin_id_key, null) == $user_id;
		}

		function get_main_admin_id() {
			return (int) get_option(VG_Admin_To_Frontend::$main_admin_id_key, null);
		}

		function set_main_admin_id($user_id = null, $overwrite = false) {
			if (!$user_id) {
				$user_id = get_current_user_id();
			}
			if ($this->get_main_admin_id() && !$overwrite) {
				return;
			}
			update_option(VG_Admin_To_Frontend::$main_admin_id_key, $user_id);
		}

		function get_settings($key = null, $default = null) {
			$options = get_option(VG_Admin_To_Frontend::$textname, array());

			$out = $options;
			if (!empty($key) && isset($options[$key])) {
				$out = $options[$key];
			}
			if (empty($out)) {
				$out = $default;
			}
			return apply_filters('vg_frontend_admin/settings', $out, $key, $default);
		}

		function get_plugin_install_url($plugin_slug) {
			$install_plugin_base_url = ( is_multisite() ) ? network_admin_url() : admin_url();
			$install_plugin_url = add_query_arg(array(
				's' => $plugin_slug,
				'tab' => 'search',
				'type' => 'term'
					), $install_plugin_base_url . 'plugin-install.php');
			return $install_plugin_url;
		}

		function init() {
			$this->args = array(
				'main_plugin_file' => __FILE__,
				'show_welcome_page' => true,
				'welcome_page_file' => VG_Admin_To_Frontend::$dir . '/views/welcome-page-content.php',
				'logo' => plugins_url('/assets/imgs/logo.png', __FILE__),
				'plugin_name' => VG_Admin_To_Frontend::$name,
				'plugin_prefix' => 'wpatof_',
				'plugin_version' => VG_Admin_To_Frontend::$version,
				'plugin_options' => get_option(VG_Admin_To_Frontend::$textname, false),
				'buy_link' => $this->get_upgrade_url(),
				'buy_link_text' => __('Try premium plugin for FREE - 7 Days'),
			);

			if (is_multisite()) {
				$this->args['buy_link_text'] .= ' (Unlimited sites)';
			}

			$this->vg_plugin_sdk = new VG_Freemium_Plugin_SDK(apply_filters('vg_admin_to_frontend/plugin_sdk_args', $this->args));

			add_shortcode('vg_display_admin_page', array($this, 'get_admin_page_for_frontend'));
			add_shortcode('vg_display_logout_link', array($this, 'get_logout_link'));
			add_shortcode('vg_display_edit_link', array($this, 'get_edit_link'));
			add_action('admin_head', array($this, 'cleanup_admin_page_for_frontend'));

			if (!WPFA_IS_EMBED_MODE) {
				add_action('admin_menu', array($this, 'register_menu_page'));
			}
			add_action('admin_page_access_denied', array($this, 'show_message_if_page_not_allowed'));
			add_action('admin_init', array($this, 'maybe_redirect_direct_link'));
			add_action('after_setup_theme', array($this, 'late_init'));
			add_action('admin_bar_menu', array($this, 'add_direct_link_menu'), 999);
			add_action('get_edit_post_link', array($this, 'add_post_edit_link'));
			add_action('wp_ajax_vg_frontend_admin_save_quick_settings', array($this, 'save_quick_settings'));

			if ($this->get_settings('hide_admin_bar_frontend') && !is_admin()) {
				add_filter('show_admin_bar', '__return_false');
			}

			$compatibility_files = $this->get_files_list(__DIR__ . '/inc/compatibility/');
			foreach ($compatibility_files as $file) {
				require_once $file;
			}
		}

		function show_message_if_page_not_allowed() {
			if (empty($_GET['vgfa_source'])) {
				return;
			}
			printf(__('Tip from WP Frontend Admin: This page requires a different user role. Please view this page as administrator and follow the instructions in the "quick settings" section to fix this. Example: %s', VG_Admin_To_Frontend::$textname), '<img src="' . esc_url(plugins_url('/assets/imgs/page-not-allowed-instructions.png', __FILE__)) . '" />');
		}

		function save_quick_settings() {
			global $wpdb;
			if (!current_user_can('manage_options') || empty($_REQUEST['ID']) || empty($_REQUEST['_wpnonce']) || !wp_verify_nonce($_REQUEST['_wpnonce'], 'vg_frontend_admin_save_quick_settings')) {
				wp_send_json_error(__('Settings couldn\'t be saved', VG_Admin_To_Frontend::$textname));
			}

			$post_id = (int) $_REQUEST['ID'];
			$update = array();

			if (!empty($_REQUEST['post_title'])) {
				$update['post_title'] = sanitize_text_field($_REQUEST['post_title']);
			}
			if (!empty($_REQUEST['page_template'])) {
				update_post_meta($post_id, '_wp_page_template', sanitize_text_field($_REQUEST['page_template']));
			}
			if (isset($_REQUEST['vgfa_hidden_elements'])) {
				update_post_meta($post_id, 'vgfa_hidden_elements', implode(', ', array_filter(explode(',', sanitize_text_field($_REQUEST['vgfa_hidden_elements'])))));
			}
			if (!empty($_REQUEST['post_name'])) {
				$update['post_name'] = sanitize_title($_REQUEST['post_name']);
			}

			if (!empty($update)) {
				$wpdb->update($wpdb->posts, $update, array('ID' => $post_id), '%s', '%d');
			}

			if (!empty($_REQUEST['menu'])) {
				$already_in_menu = new WP_Query(array(
					'post_type' => 'nav_menu_item',
					'posts_per_page' => 1,
					'meta_query' => array(
						array(
							'key' => '_menu_item_object_id',
							'value' => $post_id
						)
					),
					'tax_query' => array(
						array(
							'taxonomy' => 'nav_menu',
							'field' => 'term_id',
							'terms' => (int) $_REQUEST['menu'],
						),
					),
				));

				if (!$already_in_menu->posts) {
					wp_update_nav_menu_item((int) $_REQUEST['menu'], 0, array(
						'menu-item-title' => get_the_title($post_id),
						'menu-item-object-id' => $post_id,
						'menu-item-object' => 'page',
						'menu-item-status' => 'publish',
						'menu-item-type' => 'post_type',
					));
				}
			}

			clean_post_cache($post_id);

			wp_send_json_success(array(
				'message' => __('Settings saved successfully. We will reload the page to show the new changes', VG_Admin_To_Frontend::$textname),
				'new_url' => get_permalink($post_id)
			));
		}

		/**
		 * Get all files in the folder
		 * @return array
		 */
		function get_files_list($directory_path, $file_format = '.php') {
			$files = glob(trailingslashit($directory_path) . '*' . $file_format);
			return $files;
		}

		function add_post_edit_link($link) {
			if (is_admin()) {
				return $link;
			}
			$add_post_edit_link = $this->get_settings('add_post_edit_link');

			if ($add_post_edit_link) {
				$page_id = $this->get_page_id(admin_url('post.php?action=edit'), __('Edit'));
				$link = esc_url(add_query_arg('post', get_the_ID(), get_permalink($page_id)));
			}
			return $link;
		}

		function get_shortcode_for_slug($slug, $only_beginning = false) {
			$out = '[vg_display_admin_page page_url="' . $slug . '"';
			if (!$only_beginning) {
				$out .= ']';
			}

			return $out;
		}

		function get_page_id($admin_url, $title) {
			global $wpdb;

			$title = preg_replace('/\d+/', '', $title);
			$shortcode = $this->get_shortcode_for_slug($admin_url, true);
			$full_shortcode = $this->get_shortcode_for_slug($admin_url);
			$page_id = (int) $wpdb->get_var("SELECT ID FROM $wpdb->posts WHERE post_type = 'page' AND post_status = 'publish' AND post_content LIKE '%" . esc_sql($shortcode) . "%' ");
			if (!$page_id) {
				$page_id = wp_insert_post(array(
					'post_type' => 'page',
					'post_status' => 'publish',
					'post_title' => $title,
					'post_content' => $full_shortcode
				));
			}
			return $page_id;
		}

		/**
		 * Redirect from wp-admin url to the real frontend url.
		 * We could link to the frontend url directly on the toolbar, but we´re doing this
		 * redirect so we create the base page only when needed.
		 * 
		 * @return null
		 */
		function maybe_redirect_direct_link() {
			if (empty($_GET['vgca_direct'])) {
				return;
			}
			$page_id = $this->get_page_id(esc_url(admin_url($_GET['vgca_slug'])), sanitize_text_field($_GET['title']));
			$page_url = get_permalink($page_id);
			wp_redirect($page_url);
			exit();
		}

		function add_direct_link_menu($wp_admin_bar) {
			$args = array(
				'id' => 'vgca-direct-frontend-link',
				'title' => __('View on the frontend', VG_Admin_To_Frontend::$textname),
				'href' => add_query_arg(array(
					'vgca_direct' => 1,
						), admin_url()),
				'meta' => array('class' => 'vgca-direct-frontend-link')
			);
			$wp_admin_bar->add_node($args);
		}

		function late_init() {
			$urls = array();
			if (function_exists('dapof_fs') && !dapof_fs()->can_use_premium_code__premium_only()) {
				$urls = array(
					admin_url('edit.php'),
					admin_url('post-new.php'),
					admin_url('post.php?action=edit'),
					admin_url('edit-tags.php?taxonomy=post_tag'),
					admin_url('edit-tags.php?taxonomy=category'),
				);
			}
			$this->allowed_urls = apply_filters('vg_admin_to_frontend/allowed_urls', $urls);
		}

		function register_menu_page() {
			add_menu_page(
					VG_Admin_To_Frontend::$name, VG_Admin_To_Frontend::$name, 'manage_options', 'wpatof_welcome_page', array($this->vg_plugin_sdk, 'render_welcome_page'), plugins_url('/assets/imgs/wp-admin-icon.png', __FILE__)
			);
		}

		function required_capability_by_current_page() {

			global $pagenow, $menu, $submenu;

			if (!is_admin() || !current_user_can('manage_options')) {
				return;
			}

			if (!empty($_GET['page'])) {
				$page_slug = sanitize_text_field($_GET['page']);
			} elseif (!empty($_GET['post_type'])) {
				$page_slug = $pagenow . '?post_type=' . sanitize_text_field($_GET['post_type']);
			} else {
				$page_slug = $pagenow;
			}

			$capability = null;
			foreach ($menu as $menu_page) {
				if (!empty($menu_page[2]) && $menu_page[2] === $page_slug) {
					$capability = $menu_page[1];
					break;
				}
			}
			if (!$capability) {
				foreach ($submenu as $submenu_items) {
					foreach ($submenu_items as $menu_page) {

						if (!empty($menu_page[2]) && $menu_page[2] === $page_slug) {
							$capability = $menu_page[1];
							break;
						}
					}
				}
			}
			return $capability;
		}

		function cleanup_admin_page_for_frontend() {
			$custom_css = preg_replace('/(\r|\n)/', '', $this->get_settings('admin_view_css', ''));
			$hidden_elements = (!empty($_GET['vgfa_source']) ) ? get_post_meta($_GET['vgfa_source'], 'vgfa_hidden_elements', true) : '';

			$required_capability_html = '';
			if (!empty($_GET['vgfa_source'])) {
				$required_capability = $this->required_capability_by_current_page();

				if ($required_capability) {
					if (!function_exists('get_editable_roles')) {
						require_once ABSPATH . 'wp-admin/includes/user.php';
					}

					$required_capability_html = '<ul>';
					foreach (get_editable_roles() as $role_name => $role_info) {
						if (!empty($role_info['capabilities']) && isset($role_info['capabilities'][$required_capability])) {
							$required_capability_html .= '<li>' . $role_name . '</li>';
						}
					}
					$required_capability_html .= '</ul>';
					$required_capability_html .= sprintf(__('<div><b>Give permission to other user roles</b>. You can use the <a href="%s" target="_blank">User Role Editor</a> plugin to assign the capability to the user role required by the page: "%s". Careful, assign advanced capabilities only if you trust the users.</div>', VG_Admin_To_Frontend::$textname), VG_Admin_To_Frontend_Obj()->get_plugin_install_url('user-role-editor'), esc_html($required_capability));
				}
			}
			?>
			<style>
				.vgca-only-admin-content body {
					background: transparent;
				}

				<?php
				// Hide elements selected on quick-settings
				if (!empty($hidden_elements)) {
					$cleaned_selectors = implode(', ', array_filter(explode(',', $hidden_elements)));
					echo $cleaned_selectors . ',';
				}
				?>				
				.vgca-only-admin-content #wpadminbar,
				.vgca-only-admin-content #adminmenumain,
				.vgca-only-admin-content #update-nag, 
				.vgca-only-admin-content .update-nag,
				.vgca-only-admin-content #wpfooter{
					display: none !important;
				}

				.vgca-only-admin-content .folded #wpcontent, 
				.vgca-only-admin-content .folded #wpfooter,
				.vgca-only-admin-content #wpcontent,
				.vgca-only-admin-content #wpfooter {
					margin-left: 0px !important;
					padding-left: 0px !important;
				}
				html.wp-toolbar.vgca-only-admin-content  {
					padding-top: 0px !important;
				}

			</style>
			<script>
				var customCss = <?php echo json_encode($custom_css); ?>;
				var wpAdminBase = <?php echo json_encode(admin_url()); ?>;
				var vgfaRequiredRoles = <?php echo json_encode($required_capability_html); ?>;

				// We add the class here, inline, to hide the admin menu and bar quicker
				// if we load it on the backend.js file it appears for a second while the file is downloaded
				if (window.parent.location.href !== window.location.href) {
					jQuery('html').addClass('vgca-only-admin-content');
				}
			</script>
			<?php
			wp_enqueue_script('vg-frontend-admin-outline', plugins_url('/assets/vendor/jQuery.DomOutline.js', VG_Admin_To_Frontend::$file), array('jquery'));
			wp_enqueue_script('vg-frontend-admin-init', plugins_url('/assets/js/backend.js', VG_Admin_To_Frontend::$file), array('jquery'));
		}

		function get_edit_link($atts = array(), $content = '') {
			extract(shortcode_atts(array(
				'post_id' => '',
							), $atts));

			if (!is_user_logged_in()) {
				return;
			}
			if (!$post_id) {
				$post_id = get_the_ID();
			}
			ob_start();
			edit_post_link(__('Edit', VG_Admin_To_Frontend::$textname), '', '', $post_id);
			$out = ob_get_clean();
			return $out;
		}

		function get_logout_link($atts = array(), $content = '') {
			extract(shortcode_atts(array(
				'page_url' => '',
							), $atts));

			if (!is_user_logged_in()) {
				return;
			}
			$logout_link = str_replace('<a ', '<a class="vg-logout-link" ', wp_loginout($_SERVER['REQUEST_URI'], false));
			$out = '<style>.vg-logout-link{padding:5px;background:#000;color:#fff;text-decoration:none}</style>' . $logout_link;
			return $out;
		}

		function get_admin_page_for_frontend($atts = array(), $content = '') {
			extract(shortcode_atts(array(
				'page_url' => '',
				'forward_parameters' => true,
				'allowed_roles' => null,
				'required_capabilities' => null,
				'allowed_user_ids' => null
							), $atts));

			if (!is_user_logged_in()) {
				$login_message = wp_kses_post(wpautop($this->get_settings('login_message')));
				$login_form = wp_login_form(array('echo' => false, 'redirect' => $_SERVER['REQUEST_URI']));

				ob_start();
				include 'views/frontend/log-in-message.php';
				return ob_get_clean();
			}

			if (!$page_url) {
				return;
			}

			$allowed_to_view = true;
			if (!empty($allowed_roles)) {
				$allowed_roles = explode(',', $allowed_roles);
				$user_data = get_userdata(get_current_user_id());
				$allowed_to_view = false;

				foreach ($allowed_roles as $allowed_role) {
					if (in_array($allowed_role, $user_data->roles)) {
						$allowed_to_view = true;
						break;
					}
				}
			}
			if (!empty($required_capabilities)) {
				$required_capabilities = explode(',', $required_capabilities);
				$user_data = get_userdata(get_current_user_id());
				$allowed_to_view = false;

				foreach ($required_capabilities as $required_capability) {
					if (current_user_can($required_capability)) {
						$allowed_to_view = true;
						break;
					}
				}
			}

			if (!empty($allowed_user_ids)) {
				$allowed_user_ids = array_map('intval', explode(',', $allowed_user_ids));
				$allowed_to_view = in_array(get_current_user_id(), $allowed_user_ids);
			}
			if (!$allowed_to_view) {
				ob_start();
				include 'views/frontend/logged-in-user-not-allowed.php';
				return ob_get_clean();
			}

			$full_url = ( strpos($page_url, '//') !== false ) ? $page_url : admin_url($page_url);
			$full_url = remove_query_arg('post', $full_url);
			if (!empty($this->allowed_urls) && !in_array($full_url, $this->allowed_urls)) {
				if (is_super_admin()) {
					ob_start();
					include 'views/frontend/wrong-plan.php';
					return ob_get_clean();
				} else {
					return;
				}
			}
			ob_start();

			if ($this->is_main_admin_id() && !$this->get_settings('disable_quick_settings')) {
				$post = get_post(get_the_ID());
				$templates = wp_get_theme()->get_page_templates();
				$current_template = get_post_meta(get_the_ID(), '_wp_page_template', true);
				$menus = get_terms('nav_menu', array('hide_empty' => false));
				include 'views/frontend/quick-settings.php';
			}
			include 'views/frontend/page.php';
			wp_enqueue_script('vg-frontend-admin-init', plugins_url('/assets/js/frontend.js', VG_Admin_To_Frontend::$file), array('jquery'));
			wp_localize_script('vg-frontend-admin-init', 'vgfa_data', array(
				'wp_ajax_url' => admin_url('admin-ajax.php')
			));
			return ob_get_clean();
		}

		/**
		 * Creates or returns an instance of this class.
		 */
		static function get_instance() {
			if (null == VG_Admin_To_Frontend::$instance) {
				VG_Admin_To_Frontend::$instance = new VG_Admin_To_Frontend();
				VG_Admin_To_Frontend::$instance->init();
			}
			return VG_Admin_To_Frontend::$instance;
		}

		function __set($name, $value) {
			$this->$name = $value;
		}

		function __get($name) {
			return $this->$name;
		}

	}

	if (!function_exists('VG_Admin_To_Frontend_Obj')) {

		function VG_Admin_To_Frontend_Obj() {
			return VG_Admin_To_Frontend::get_instance();
		}

	}

	VG_Admin_To_Frontend_Obj();


	require 'inc/options-init.php';
}
